var _ = require('lodash');

function determineName() {
  var url = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var itemName = arguments[1];
  var layerName = arguments[2];
  var layers = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

  if (/(feature|map)server\/\d+/i.test(url)) {
    return itemName;
  } else if (layerName && layers.length >= 2) {
    return layerName;
  } else {
    return itemName;
  }
}

function handleFail(item, e) {
  console.log('error getting layers');
  var dataset = composeDataset(item);
  dataset.set('serviceDown', true);
  return dataset;
}

function determineSource(metadata, item) {
  var sourceProvenance1 = 'metadata.dataIdInfo.idCitation.citRespParty.rpOrgName';
  var sourceProvenance2 = 'metadata.dataIdInfo.idPoC.rpOrgName';
  var result = {};
  var source1 = _.get(metadata, sourceProvenance1, '');
  var source2 = _.get(metadata, sourceProvenance2, '');
  if (source1) {
    result.source = source1;
    result.sourceProvenance = sourceProvenance1;
  } else if (source2) {
    result.source = source2;
    result.sourceProvenance = sourceProvenance2;
  } else {
    result.source = item.owner;
    result.sourceProvenance = 'item.owner';
  }
  return result;
}

var common = {
  determineName: determineName,
  handleFail: handleFail,
  determineSource: determineSource
};

var hubTypeMap = {
  'Document Link': 'Document',
  'Feature Service': 'Map',
  'Map Service': 'Map',
  'Feature Service Layer': 'Dataset',
  'Web Map': 'Map',
  'Image Service': 'Raster Dataset',
  'Vector Tile Service': 'Map Layer',
  'Web Scene': 'Map',
  'Scene Service': 'Map Layer',
  'Feature Collection': 'File',
  'Feature Collection Template': 'Template',
  'Layer Package': 'File',
  'City Engine Web Scene': 'Map',
  'Web Mapping Application': 'Application',
  'Explorer Add In': 'Other',
  'Operations Dashboard Add In': 'Other',
  'Service Definition': 'Other',
  'Mobile Map Package': 'File',
  'Map Package': 'File',
  'Layer Template': 'Template',
  'Geoprocessing Service': 'Geoprocessing',
  'File Geodatabase': 'Spatial File',
  'CSV Collection': 'File',
  'Microsoft Excel': 'Document',
  'Shapefile': 'Spatial File',
  'GeoJSON': 'Spatial File',
  'KML Collection': 'Spatial File',
  'PDF': 'Document',
  'Hub Intitiative': 'Hub Content',
  'Hub Page': 'Hub Content',
  'Hub Site': 'Hub Content',
  'Code Sample': 'Code',
  'CAD Drawing': 'Document',
  'Desktop Application': 'Application',
  'Desktop Application Template': 'Template',
  'Desktop Style': 'File',
  'Geoprocessing Package': 'Geoprocessing',
  'Geoprocessing Sample': 'Geoprocessing',
  'Image': 'Document',
  'iWork Keynote': 'Document',
  'iWork Numbers': 'Document',
  'Layer File': 'Spatial File',
  'Layout': 'Other',
  'Locator Package': 'Other',
  'Map Service Definition': 'Template',
  'Map Template': 'Template',
  'Microsoft Powerpoint': 'Document',
  'Microsoft Visio': 'Document',
  'Pro Map': 'Spatial File',
  'Project Package': 'Other',
  'Project Template': 'Template',
  'Raster Function Template': 'Template',
  'Rule Package': 'Other',
  'Vector Tile Package': 'Spatial File',
  'Workflow Manager Package': 'Other',
  '360 VR Experience': 'Other',
  'Stream Service': 'Dataset',
  'Application Configuration': 'Template',
  'Map Service Layer': 'Dataset',
  'Web Map Tile Service': 'Map Layer',
  'Web Map Service': 'Map'
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};





















var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

function extractErrorFields(e) {
  return _extends({}, {
    message: e.message,
    name: e.name,
    stack: e.stack,
    timestamp: Date.now(),
    url: null,
    statusCode: null,
    httpMethod: null
  });
}

function composeDataset(components) {
  var layer = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  try {
    var ds = _extends({}, components);
    ds.layers = layer.id !== undefined ? [layer] : components.layers; // if this is indeed a 'layer' dataset, then set layers = [this layer] else it means this is a consolidated dataset and must have all the layers
    ds.default = {
      id: layer.id !== undefined // Have to do this check because number 0 ? option 1 : option 2 produces output option 2! and layer id could be 0
      ? components.item.id + '_' + layer.id : components.item.id,
      description: layer.description || components.item.description,
      extent: components.item.extent,
      type: determineHubType(components.item.type, layer.type),
      name: common.determineName(components.item.url, components.item.name || components.item.title, layer.name, components.layers),

      spatialReference: layer.extent && layer.extent.spatialReference.wkid || components.item.spatialReference,
      composedAt: Date.now(),
      url: determineDirectURL(components.item, layer),
      source: common.determineSource(components.metadata, components.item),
      groupIds: groupsArray(components.groups),
      composeStatus: {
        pass: true
      },
      deleted: false
    };
    return ds;
  } catch (e) {
    console.log(e);
    var _ds = _extends({}, components);
    _ds.default = {
      composeStatus: {
        pass: false
      },
      id: layer.id ? components.item.id + '_' + layer.id : components.item.id,
      deleted: false
    };
    _ds.errors = [];
    _ds.errors.push(_extends({}, extractErrorFields(e), { field: 'default' }));
    return _ds;
  }
}

function groupsArray(groups) {
  var member = groups.member.map(function (g) {
    return g.id;
  });
  var admin = groups.admin.map(function (g) {
    return g.id;
  });
  var other = groups.other.map(function (g) {
    return g.id;
  });

  return member.concat(admin).concat(other);
}

function determineDirectURL(item) {
  var layer = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var url = item.url;
  // TODO: Remove the following *if condition* code because it changes feature service url which affects stats calculation in the hub indexers
  if (item.url && typeof layer.id === 'number') {
    url = item.url + '/' + layer.id;
  }
  return url;
}

function determineHubType(itemType, layerType) {
  if (layerType) return 'Dataset';else if (hubTypeMap[itemType]) return hubTypeMap[itemType];else return 'Other';
}

function composeDatasets(assembled) {
  var datasets = [composeDataset(assembled)];
  if (!assembled.layers) {
    return datasets;
  }
  assembled.layers.forEach(function (layer) {
    datasets.push(composeDataset(assembled, layer));
  });
  return datasets;
}

require('isomorphic-fetch'); // isomorphic-fetch creates a global 'fetch' function
var fastXmlParser = require('fast-xml-parser');
var opts = {
  // options for fastXmlParser to read tag attrs
  ignoreTextNodeAttr: false,
  attrPrefix: '@_', // attr name will be a new field in the resulting json with this prefix
  textNodeName: '#value' // the resulting json will have field called #value pointing to the actual tag value
};

var req = function (url) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return fetch(url, options) // eslint-disable-line
  .then(function (response) {
    if (response.ok) {
      if (options && options.xml) {
        return response.text().then(function (xml) {
          return fastXmlParser.parse(xml.toString(), opts);
        });
      } else {
        return response.json();
      }
    } else {
      throw { // eslint-disable-line
        message: response.status + ' ' + response.statusText + ' ' + response.url,
        url: response.url,
        statusCode: response.status,
        name: 'HTTPError',
        timestamp: Date.now(),
        stack: response.status + ' ' + response.statusText + ' ' + response.url,
        httpMethod: options.method || 'get'
      };
    }
  }).then(function (json) {
    // do the json check dance to the ArcGis tunes
    if (json && json.error) {
      throw { // eslint-disable-line
        message: json.error.code + ' ' + json.error.messageCode + ' ' + json.error.message + ' ' + url,
        url: url,
        statusCode: json.error.code,
        name: 'HTTPError',
        timestamp: Date.now(),
        stack: json.error.code + ' ' + json.error.messageCode + ' ' + json.error.message + ' ' + url,
        httpMethod: options.method || 'get'
      };
    }
    return json;
  }).catch(function (err) {
    return Promise.reject(err);
  });
};

var fetchItem = function (baseUrl, itemId) {
  return req(baseUrl + '/sharing/rest/content/items/' + itemId + '?f=json'); // eslint-disable-line
};

var fetchMetadata = function (baseUrl, itemId) {
  return req(baseUrl + '/sharing/rest/content/items/' + itemId + '/info/metadata/metadata.xml', { xml: true }); // eslint-disable-line
};

var fetchServer = function (url) {
  return req(url); // eslint-disable-line
};

var fetchItemGroups = function (host, id) {
  var url = host + '/sharing/content/items/' + id + '/groups?f=json';
  return req(url) // eslint-disable-line
  .catch(function (e) {
    return Promise.reject(e);
  });
};

var fetchLayers = function (url) {
  return req(url) // eslint-disable-line
  .then(function (layers) {
    // return layers field which is an array
    return layers.layers;
  }).catch(function (e) {
    return Promise.reject(e);
  });
};

var RSVP = require('rsvp');
var allowableFieldsWithErrors = ['metadata'];

function compose(params) {
  var itemPromise = {
    item: fetchItem(params.baseUrl, params.id)
  };
  return RSVP.hashSettled(itemPromise).then(assemblePromiseResults).then(function (itemResult) {
    if (!shouldProceed(itemResult)) {
      // It is a fatal error like may be the item itself is not available
      throw _extends({}, itemResult, {
        default: {
          id: params.id,
          composeStatus: {
            pass: false
          }
        }
      });
    }
    return itemResult.item;
  }).then(function (item) {
    return fetchParts(item, params);
  }).then(function (assembled) {
    // An assembled dataset has the constituent parts fetched form arcgis and errors if any.
    // We may want to create multiple datasets based on whether it has layers
    // and also each dataset must have default field
    return shouldProceed(assembled) ? composeDatasets(assembled) : _extends({}, assembled, {
      default: {
        id: params.id,
        composeStatus: {
          pass: false
        },
        deleted: false
      }
    });
  }).catch(function (e) {
    // e is always of the dataset format but will contain errors array
    return Promise.resolve([e]);
  }).then(function (results) {
    // Finally, update the composeStatus with 'retries' number and the timestamp when last compose step was retried
    results.forEach(function (result) {
      result.default.composeStatus.retries = params.retries;
      result.default.composeStatus.lastRetriedAt = Date.now();
    });
    return results;
  });
}

function fetchParts(item, params) {
  /*
    Source url:
    item: {agoHost}/sharing/rest/content/items/{id}?f=json
    metadata: {agoHost}/sharing/rest/content/items/{itemId}/info/metadata/metadata.xml
    groups: {agoHost}/sharing/content/items/{id}/groups?f=json
    server: {item.url}?f=json
    layers: {item.url}/layers?f=json
  */
  var promises = {
    item: Promise.resolve(item),
    metadata: fetchMetadata(params.baseUrl, params.id),
    groups: fetchItemGroups(params.baseUrl, params.id)
  };
  if (/(feature|map|image)server(\/\d*)?/i.test(item.url)) {
    promises.server = fetchServer(item.url + '?f=json');
  }
  if (/(feature|map)server(\/\d*)?/i.test(item.url)) {
    promises.layers = fetchLayers(item.url + '/layers?f=json');
  }
  // TODO: Fetch item data where appropriate
  return RSVP.hashSettled(promises).then(assemblePromiseResults);
}

function assemblePromiseResults(promiseResults) {
  /*
    RSVP gives us the power to inspect the promise result for any errors and take action
    This function returns an assembled json of all the individual components of a dataset
    and if there is an error in any of these steps, we create a new field called errors which is an array
    and add the error details object to that array
    {
      item:
      metadata:
      groups:
      server:
      layers:
      errors:
    }
  */
  var assembled = { errors: [] };
  Object.entries(promiseResults).forEach(function (_ref) {
    var _ref2 = slicedToArray(_ref, 2),
        key = _ref2[0],
        val = _ref2[1];

    if (val.state === 'fulfilled') {
      assembled[key] = val.value;
    } else {
      // means state is 'rejected'
      assembled[key] = {}; // we don't want anything in the field if it has errored out
      assembled.errors.push(_extends({}, val.reason, { field: key }));
      // val.reason will be an error like object - check out request.js. It could even be a native nodejs error
    }
  });
  return assembled;
}

function shouldProceed(assembled) {
  // if there is an error obj whose field is not in the allowableFieldsWithErrors, then we should not proceed
  return assembled.errors.filter(function (error) {
    return !allowableFieldsWithErrors.includes(error.field);
  }).length === 0;
}

var validateObjectIdField = function (layer) {
  var report = {
    pass: true,
    name: 'ObjectIdFieldValidator'
    // logic is to test for an array of fields of all the layers with the required type.
    // findWhere function checks and returns the first occurence
  };var oidField = layer.fields.filter(function (field) {
    return field.type === 'esriFieldTypeOID';
  })[0];

  if (!oidField) {
    report.pass = false;
    report.message = {
      type: 'error',
      log: 'Layer or table does not have an ObjectId field (a field of type esriFieldTypeOID)',
      i18n: {
        key: 'messages.validators.objectidfield.missing'
      }
    };
  }
  return Promise.resolve(report);
};

var validateRecordCount = function (layer) {
  var report = {
    pass: true,
    name: 'RecordCountValidator'
  };
  if (layer.maxRecordCount > 5000) {
    report.message = {
      type: 'warn',
      log: 'The maxRecordCount for this layer is greater than 5000. This can cause browsers to crash. We recommend lowering this number.',
      i18n: {
        key: 'messages.validators.record_count_query.more_than_5000'
      }
    };
  }
  if (layer.maxRecordCount > 10000) {
    // create an error
    report.pass = false;
    report.message = {
      type: 'error',
      log: 'The maxRecordCount for this layer is greater than 10000. This is too high for the majority of browsers to handle. This dataset can not be included in Open Data.',
      i18n: {
        key: 'messages.validators.record_count_query.max_record_count_too_high'
      }
    };
  }

  var promise = Promise.resolve(report);
  return promise;
};

var validateDisplayField = function (layer) {
  var report = {
    pass: true,
    name: 'DisplayFieldValidator'
  };
  var field = void 0;
  if (!layer.displayField) {
    field = getFallBackField(layer.fields);
    report.data = {
      displayField: field.name
    };
  } else {
    // Ensure that the field is actually in the fields array
    var chk = layer.fields.filter(function (field) {
      return field.name === layer.displayField;
    })[0];
    if (!chk) {
      field = getFallBackField(layer.fields);
      report.data = {
        displayField: field.name
      };
      report.message = {
        type: 'warn',
        log: 'Specified display field ' + layer.displayField + ' was not present in the dataset. Using ' + layer.fields[0] + ' as display field instead',
        i18n: {
          key: 'messages.validators.display_field.not_present',
          data: {
            field_name: layer.displayField
          }
        }
      };
    }
  }

  return Promise.resolve(report);
};

function getFallBackField(fields) {
  var field = fields.filter(function (field) {
    return field.type === 'esriFieldTypeString';
  })[0];

  return field;
}

var validateIdsOnlyQuery = function (layer) {
  var report = {
    pass: true,
    name: 'IdsOnlyQueryValidator'
  };
  var oidField = layer.fields.filter(function (layer) {
    return layer.type === 'esriFieldTypeOID';
  })[0];
  if (!layer.objectIdField && !oidField) {
    report.pass = false;
    report.data = {
      idsOnlyQuerySupported: false
    };
    report.message = {
      type: 'warn',
      log: 'Ids-only-query is not supported because no field is designated to be ObjectIdField.',
      i18n: {
        key: 'messages.validators.idsonlyquery.unsupported',
        data: null
      }
    };
  }
  return Promise.resolve(report);
};

var validateLastEditedDate = function (layer) {
  var report = {
    pass: true,
    name: 'LastEditedDateValidator',
    data: {
      last_edited_date_enabled: false
    }
  };

  if (layer.currentVersion && layer.currentVersion >= 10.1) {
    var lastEditDate = false;
    var lastEditedDateField = void 0;
    if (layer.editingInfo && layer.editingInfo.lastEditDate) {
      lastEditDate = true;
      report.last_edited_date_enabled = true;
      report.last_edited_date = new Date(layer.editingInfo.lastEditDate).toISOString();
    } else if (layer.editFieldsInfo) {
      lastEditedDateField = layer.editFieldsInfo.editDateField;
      var countOfLastEditedDateFields = layer.fields.filter(function (field) {
        return field.name === 'lastEditedDateField';
      }).length;
      if (countOfLastEditedDateFields === 0) {
        report.message = {
          type: 'warn',
          log: 'Although the service reports that editor tracking is enabled, the date field specified (' + lastEditedDateField + ') is not present in the layer.',
          i18n: {
            key: 'messages.validators.last_edited_date.field_missing',
            data: {
              field_name: lastEditedDateField
            }
          }
        };
        lastEditedDateField = '';
      }
    } else {
      var countOfFieldDates = layer.fields.filter(function (layer) {
        return layer.type === 'esriFieldTypeDate' && layer.name === 'last_edited_date';
      }).length;
      if (countOfFieldDates) {
        lastEditedDateField = 'last_edited_date';
      }
    }
    if (lastEditedDateField) {
      getLastEditedDate(lastEditedDateField, layer.url) // eslint-disable-line
      .then(function (lastEditedDate) {
        report.data.last_edited_date = lastEditedDate;
        report.data.last_edited_date_enabled = true;
      }).fail(function (rejection) {
        console.log('LastEditedDateValidator xhr failed ' + rejection.message.log);
        report.message = {
          type: 'warn',
          log: 'An error occurred getting the MAX value from the ' + lastEditedDateField + " field. Using the item's updated timestamp for this dataset.",
          i18n: {
            key: 'messages.validators.last_edited_date.query_error',
            data: {
              field_name: lastEditedDateField
            }
          }
        };
        report.data.last_edited_date_enabled = false;
      });
    } else {
      if (lastEditDate === false) {
        report.message = {
          log: 'In order to accurately show when your data is updated, editor tracking must be enabled. See here for documentation.',
          i18n: {
            key: 'messages.validators.last_edited_date.not_enabled'
          }
        };
      }
    }
  } else {
    report.message = {
      log: 'Editor tracking is not enabled on your server. Please upgrade to version 10.1 or higher.',
      i18n: {
        key: 'message.validators.last_edited_date.down_level'
      },
      type: 'info'
    };
  }
  return Promise.resolve(report);
};

var validateLicense = function (item) {
  var report = {
    pass: true,
    name: 'LicenseValidator'
  };
  if (!item.licenseInfo || item.licenseInfo === '') {
    report.message = {
      type: 'warn',
      log: 'Item does not have a licence specified.',
      i18n: {
        key: 'messages.validators.no_license.item',
        data: {
          target: '_blank'
        }
      }
    };
  }
  return Promise.resolve(report);
};

var itemTypesThatNeedExtents = ['feature service', 'map service', 'image service', 'web map'];

var validateExtent = function (item) {
  var report = {
    pass: true,
    name: 'ExtentValidator'
  };

  if (shouldItemHaveExtent(item.type)) {
    if (!item.extent || item.extent.length === 0) {
      report.message = {
        log: 'Item does not have an extent',
        type: 'warn',
        i18n: {
          key: 'messages.processors.item.no_extent',
          data: {
            target: '_blank'
          }
        }
      };
    }
  }

  return Promise.resolve(report);
};

function shouldItemHaveExtent(itemType) {
  return itemTypesThatNeedExtents.includes(itemType.toLowerCase());
}

function baseStatsField(field, stats) {
  var outStats = [];
  var fieldName = field.name;
  stats.forEach(function (stat) {
    outStats.push({
      statisticType: stat,
      onStatisticField: fieldName,
      outStatisticFieldName: fieldName.toLowerCase() + '_' + stat.toLowerCase()
    });
  });
  return outStats;
}

var stats = ['sum', 'avg', 'count', 'min', 'max', 'stddev', 'var'];

var numericStatsFieldsAllLayers = {};

function getNumericStatsField(dataset) {
  var promises = [];
  dataset.layers.forEach(function (layer) {
    promises.push(getStatsForEachLayer(layer, dataset.datasetUrl));
  });
  return Promise.all(promises).then(function (layersWithStats) {
    return numericStatsFieldsAllLayers;
  });
}

function getStatsForEachLayer(layer, featureServiceUrl) {
  var numericStatsFields = [];
  numericStatsFieldsAllLayers[layer.id] = numericStatsFields;
  var integerFieldNames = layer.fields.filter(function (field) {
    return field.type === 'esriFieldTypeInteger';
  });
  var promises = [];
  var outStatistics = void 0;
  var url = void 0;

  integerFieldNames.forEach(function (field) {
    outStatistics = baseStatsField(field, stats);
    url = featureServiceUrl + '/' + layer.id.split('_')[1] + '/query?outStatistics=' + JSON.stringify(outStatistics) + '&f=json';
    numericStatsFields.push({
      meta: { url: url }
    });
    promises.push(req(url));
  });

  return Promise.all(promises).then(function (allFieldsStats) {
    // allFieldsStats and integerFieldNames are in the same order
    allFieldsStats.forEach(function (eachFieldStats, idx) {
      if (eachFieldStats.features.length > 0) {
        numericStatsFields[idx].meta.pass = true;
        numericStatsFields[idx].statistics = eachFieldStats.features[0].attributes;
      } else {
        numericStatsFields[idx].meta.pass = false;
        numericStatsFields[idx].meta.missingStats = stats;
      }
    });
  }).catch(function (rejection) {
    console.log(rejection);
  });
}

var stats$1 = ['count'];

var stringStatsFieldsAllLayers = {};

function getStringStatsField(dataset) {
  var promises = [];
  dataset.layers.forEach(function (layer) {
    promises.push(getStatsForEachLayer$1(layer, dataset.datasetUrl));
  });
  return Promise.all(promises).then(function (layersWithStats) {
    return stringStatsFieldsAllLayers;
  });
}

function getStatsForEachLayer$1(layer, featureServiceUrl) {
  var stringStatsFields = [];
  stringStatsFieldsAllLayers[layer.id] = stringStatsFields;
  var stringFieldNames = layer.fields.filter(function (field) {
    return field.type === 'esriFieldTypeString';
  });
  var promises = [];
  var outStatistics = void 0;
  var url = void 0;

  stringFieldNames.forEach(function (field) {
    outStatistics = baseStatsField(field, stats$1);
    url = featureServiceUrl + '/' + layer.id.split('_')[1] + '/query?outStatistics=' + JSON.stringify(outStatistics) + '&f=json';
    stringStatsFields.push({
      meta: { url: url }
    });
    promises.push(req(url));
  });

  return Promise.all(promises).then(function (allFieldsStats) {
    // allFieldsStats and stringFieldNames are in the same order
    allFieldsStats.forEach(function (eachFieldStats, idx) {
      if (eachFieldStats.features.length > 0) {
        stringStatsFields[idx].meta.pass = true;
        stringStatsFields[idx].statistics = eachFieldStats.features[0].attributes;
      } else {
        stringStatsFields[idx].meta.pass = false;
        stringStatsFields[idx].meta.missingStats = stats$1;
      }
    });
  }).catch(function (rejection) {
    console.log(rejection);
  });
}

var standardLicenses = [{
  type: 'CC0-1.0',
  abbr: 'CC0',
  name: 'Public Domain Dedication',
  url: 'http://creativecommons.org/publicdomain/zero/1.0'
},

// if they provide the abbreviation without a version it will default to CC-BY-4.0
{
  type: 'CC-BY-4.0',
  abbr: 'CC BY',
  name: 'Attribution 4.0 International',
  url: 'http://creativecommons.org/licenses/by/4.0'
}, {
  type: 'CC-BY-3.0',
  abbr: 'CC BY 3.0',
  name: 'Attribution 3.0 Unported',
  url: 'http://creativecommons.org/licenses/by/3.0'
}, {
  type: 'CC-BY-2.5',
  abbr: 'CC BY 2.5',
  name: 'Attribution 2.5 Generic',
  url: 'http://creativecommons.org/licenses/by/2.5'
}, {
  type: 'CC-BY-2.0',
  abbr: 'CC BY 2.0',
  name: 'Attribution 2.0 Generic',
  url: 'http://creativecommons.org/licenses/by/2.0'
}, {
  type: 'CC-BY-1.0',
  abbr: 'CC BY 1.0',
  name: 'Attribution 1.0 Generic',
  url: 'http://creativecommons.org/licenses/by/1.0'
},

// if they provide the abbreviation without a version it will default to CC-BY-SA-4.0
{
  type: 'CC-BY-SA-4.0',
  abbr: 'CC BY-SA',
  name: 'Attribution-ShareAlike 4.0 International',
  url: 'http://creativecommons.org/licenses/by-sa/4.0'
}, {
  type: 'CC-BY-SA-3.0',
  abbr: 'CC BY-SA 3.0',
  name: 'Attribution-ShareAlike 3.0 Unported',
  url: 'http://creativecommons.org/licenses/by-sa/3.0'
}, {
  type: 'CC-BY-SA-2.5',
  abbr: 'CC BY-SA 2.5',
  name: 'Attribution-ShareAlike 2.5 Generic',
  url: 'http://creativecommons.org/licenses/by-sa/2.5'
}, {
  type: 'CC-BY-SA-2.0',
  abbr: 'CC BY-SA 2.0',
  name: 'Attribution-ShareAlike 2.0 Generic',
  url: 'http://creativecommons.org/licenses/by-sa/2.0'
}, {
  type: 'CC-BY-SA-1.0',
  abbr: 'CC BY-SA 1.0',
  name: 'Attribution-ShareAlike 1.0 Generic',
  url: 'http://creativecommons.org/licenses/by-sa/1.0'
}, {
  type: 'PDDL-1.0',
  abbr: 'PDDL',
  name: 'ODC Public Domain Dedication and License',
  url: 'http://opendatacommons.org/licenses/pddl/summary'
}, {
  type: 'ODbL-1.0',
  abbr: 'ODbL',
  name: 'ODC Open Database License',
  url: 'http://opendatacommons.org/licenses/odbl/summary'
}, {
  type: 'ODC-BY-1.0',
  abbr: 'ODC BY',
  name: 'ODC Attribution License',
  url: 'http://opendatacommons.org/licenses/by/summary'
}];

var sanitize = require('sanitize-html');

/**
* Regular Expression for validating URLs
* based on http://mathiasbynens.be/demo/url-regex, https://gist.github.com/dperini/729294
*/
var validUrlWithSchemeRegEx = /^(https?|ftp):\/\/(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?$/i;

function getLicense(dataset) {
  var licenseInfo = getStructuredLicense(dataset.licenseInfo);
  return Promise.resolve(licenseInfo);
}

function getStructuredLicense(rawLicense) {
  // default structured license is none
  var license = {
    type: 'none'
  };

  if (rawLicense) {
    // ok - we have something in the license, so
    // let's sanitize it and now assume it's a custom license.

    // remove everything but links w/ href so we can poke at the text cleanly
    var sanitizedLicense = sanitize(rawLicense, {
      allowedTags: ['a'],
      allowedAttributes: { a: ['href', 'target'] }
    });

    license.type = 'custom';
    license.text = sanitizedLicense;

    var hasStandardLicense = false;

    // check for all the standard licenses in the string
    if (sanitizedLicense) {
      standardLicenses.forEach(function (stdLicense) {
        if (sanitizedLicense.toLowerCase().indexOf(stdLicense.name.toLowerCase()) > -1 || sanitizedLicense.toLowerCase().indexOf(stdLicense.url.toLowerCase()) > -1 || sanitizedLicense.toLowerCase().indexOf(stdLicense.abbr.toLowerCase()) > -1) {
          hasStandardLicense = true;
          license.type = stdLicense.type;
          license.url = stdLicense.url;
          license.abbr = stdLicense.abbr;
        }
      });

      // if we still have not found anything, lets check if we have a naked link
      if (!hasStandardLicense) {
        // check for naked url
        if (validUrlWithSchemeRegEx.test(rawLicense)) {
          license.type = 'custom';
          license.url = rawLicense;
          delete license.text; // remove the text - we only want the url
        }
        // check for naked link
        if (/^<a .+<\/a>$/.test(sanitizedLicense)) {
          // ok - it's just the anchor tag, lets pull the href and use that as the url
          var regex = /href="(.*?)"/;
          var href = regex.exec(sanitizedLicense)[1];
          license.type = 'custom';
          license.url = href;
          delete license.text; // remove the text - we only want the url
        }
      }
    }
  }

  return license;
}

function getExtendedTags(dataset, params) {
  var extendedTags = {};
  var promises = [];
  dataset.tags.forEach(function (tag) {
    promises.push(getTermExpansions(tag, params));
  });
  return Promise.all(promises).then(function (allResults) {
    dataset.tags.forEach(function (tag, idx) {
      extendedTags[tag] = allResults[idx];
    });
    return extendedTags;
  });
}

function getTermExpansions(term, params) {
  return req(params.url + term);
}

function getReverseGeocodes(params) {
  // sample url =
  // http://34.229.180.217:3000/polygons_index?extent=[[14.5910270518,45.874771227],[14.7517924328,46.0111453313]]
  return req(params.url);
}

var _$1 = require('lodash');

/**
 * Get quality score from the dataset's original dataset metadata and optionally
 * the composed default metadata, which is helpful for better measuring single layers.
 * @param  {object} dataset       AGOL dataset metadata
 * @param  {object} [composedDefault={}] composed default dataset metadata
 * @return {number}               quality score (0 - 100)
 */
function getQuality(dataset) {
  var composedDefault = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  // baseline quality score
  var total = 30;

  try {
    // Name rules (10):
    // +5 if the name doesn't include underlines
    // +5 if the name includes more than 2 words

    var name = composedDefault.name || dataset.name || dataset.title;

    if (name && !_$1.includes(name, '_')) {
      total += 5;
    }

    if (name && name.split(/\s+/).length > 2) {
      total += 5;
    }

    // Description rules (20):
    // +10 if the description exists
    // +5 if the description is longer than 500 chars
    // +5 if the description has links

    var description = composedDefault.description || dataset.description;

    if (description) {
      total += 10;

      if (description.length > 500) {
        total += 5;
      }

      if (description.includes('http')) {
        total += 5;
      }
    }

    // License rules (15):
    // +10 if the license exists
    // +3 if the license is CC0 or CC-BY
    // +2 if the license has links

    if (dataset.licenseInfo) {
      total += 10;

      if (dataset.licenseInfo.match(/cc0|cc-by/i)) {
        total += 3;
      }

      if (dataset.licenseInfo.includes('http')) {
        total += 2;
      }
    }

    // Update rules (10):
    // +10 if the dataset has been updated
    if (dataset.created && dataset.modified > dataset.created) {
      total += 10;
    }

    // Tag rules (10):
    // +4 if the dataset has more than 2 tags
    // +6 if the dataset has fewer than 5 tags

    if (dataset.tags) {
      if (dataset.tags.length > 2) {
        total += 4;
      }

      if (dataset.tags.length > 0 && dataset.tags.length <= 5) {
        total += 6;
      }
    }

    // Layer rules (5):
    // +5 if the dataset is a single layer (url ends with a number)

    var url = composedDefault.url || dataset.url;

    if (url && url.match(/\/\d+$/)) {
      total += 5;
    }

    // Field rules (10):
    // +10 if dataset fields all have an alias different from its name

    if (dataset.fields && _$1.every(dataset.fields, differentNameAlias)) {
      total += 10;
    }

    return Promise.resolve(total);
  } catch (err) {
    return Promise.reject(err);
  }
}

function differentNameAlias(field) {
  return field.name !== field.alias;
}

/**
 * Get a coverage description for an item metadata
 * @param  {object} dataset       AGOL dataset metadata
 * @return {string}               coverage description: unknown, local, regional, national, global
 */
function getCoverage(dataset) {
  var coverage = 'unknown';

  try {
    if (dataset.extent) {
      // dataset.extent = [[w,s],[e,n]]
      var latExtent = dataset.extent[1][0] - dataset.extent[0][0];
      var lngExtent = dataset.extent[1][1] - dataset.extent[0][1];
      var maxExtent = latExtent < lngExtent ? lngExtent : latExtent;

      if (maxExtent < 0 || Number.isNaN(maxExtent)) {
        coverage = 'unknown';
      } else if (maxExtent < 1) {
        coverage = 'local';
      } else if (maxExtent < 10) {
        coverage = 'regional';
      } else if (maxExtent < 50) {
        coverage = 'national';
      } else {
        coverage = 'global';
      }
    }

    return Promise.resolve(coverage);
  } catch (err) {
    return Promise.reject(err);
  }
}

var index = {
  composeDatasets: composeDatasets,
  composeDataset: composeDataset,
  compose: compose,
  fetchItem: fetchItem,
  fetchServer: fetchServer,
  fetchItemGroups: fetchItemGroups,
  fetchLayers: fetchLayers,
  fetchMetadata: fetchMetadata,
  validateObjectIdField: validateObjectIdField,
  validateRecordCount: validateRecordCount,
  validateDisplayField: validateDisplayField,
  validateIdsOnlyQuery: validateIdsOnlyQuery,
  validateLastEditedDate: validateLastEditedDate,
  validateLicense: validateLicense,
  validateExtent: validateExtent,
  enrichNumericStatsField: getNumericStatsField,
  enrichStringStatsField: getStringStatsField,
  enrichLicense: getLicense,
  enrichExtendedTags: getExtendedTags,
  enrichReverseGeocodes: getReverseGeocodes,
  enrichQuality: getQuality,
  enrichCoverage: getCoverage
};

export default index;
